.SysExp {
  width: 100%;
  height: 100%;
  display: "flex";
}

.cosapp-explorer > div {
  height: 100%;
}

.MainWindow {
  height: calc(100% - 48px);
  margin-top: 48px;
  flex: 1 1 auto;
}

.hideDiv {
  display: none;
}

.hideByPos {
  position: absolute !important;
  top: -9999px !important;
  right: -9999px !important;
}

.showDiv {
  width: 100%;
  height: 100%;
  padding: 8px 8px 0px 8px;
}

canvas {
  width: 100%;
  height: 100%;
}
