"""
Information about the frontend package of the widgets.

Until decided otherwise, the frontend version is synchronized with the backend.
"""
from ._version import __version__

module_name = "cosapp_lab"
module_version = __version__
