.cosapp-sidecar {
  background: var(--jp-layout-color0);
  overflow-y: auto;
}

.jp-SideBar.jp-mod-right > .lm-TabBar-tabCloseIcon {
  padding: 4px 0px 4px 4px;
  background-size: 16px;
  height: 16px;
  width: 16px;
  background-image: var(--jp-icon-close);
  background-position: center;
  background-repeat: no-repeat;
  align-self: center;
}

.jp-SideBar.jp-mod-right > .lm-TabBar-tabCloseIcon:hover {
  background-image: var(--jp-icon-close-circle);
}
